import re

from cy_ioc_extract.constants import ALL_EXTRACT_FIELDS, ERRORS, ALL_REGEX
from cy_ioc_extract.exception import UnsupportedExtractFieldError


class IOCEXtract:
    def __init__(self, raw_text, extract_fields=None, validate_ioc=True):
        self.validate = validate_ioc
        self.raw_text = raw_text
        self.extract_fields = extract_fields or ALL_EXTRACT_FIELDS
        self.validate_extract_fields(self.extract_fields)

    def extract_ioc(self):
        extracted_data = {ioc_type: list(set(re.findall(ALL_REGEX[ioc_type]["regex"], self.raw_text, flags=re.IGNORECASE))) for ioc_type in self.extract_fields}
        if self.validate:
            self.validate_ioc(extracted_data)
        return extracted_data

    @staticmethod
    def validate_extract_fields(extract_fields):
        invalid_items = set(extract_fields) - set(ALL_EXTRACT_FIELDS)
        if invalid_items:
            raise UnsupportedExtractFieldError(
                ERRORS.get("invalid_field_to_extract") % (str(invalid_items), str(set(ALL_EXTRACT_FIELDS))))

    @staticmethod
    def validate_ioc(extracted_data):
        for ioc_type in extracted_data:
            if extracted_data[ioc_type] and ALL_REGEX[ioc_type].get("validator"):
                extracted_data[ioc_type] = ALL_REGEX[ioc_type].get("validator")(extracted_data[ioc_type])
